@@ -1,15 +1,14 @@
 use ethers_core::types::Address;
 use std::path::PathBuf;
 
-use anyhow::{anyhow, Result};
-use cfg_if::cfg_if;
+use eyre::Result;
 use inflector::Inflector;
 use proc_macro2::{Ident, Literal, Span, TokenStream};
 use quote::quote;
 
 use syn::{Ident as SynIdent, Path};
 
-/// Expands a identifier string into an token.
+/// Expands a identifier string into a token.
 pub fn ident(name: &str) -> Ident {
     Ident::new(name, Span::call_site())
 }
@@ -22,6 +21,37 @@ pub fn safe_ident(name: &str) -> Ident {
     syn::parse_str::<SynIdent>(name).unwrap_or_else(|_| ident(&format!("{}_", name)))
 }
 
+///  Converts a `&str` to `snake_case` `String` while respecting identifier rules
+pub fn safe_snake_case(ident: &str) -> String {
+    safe_identifier_name(ident.to_snake_case())
+}
+
+///  Converts a `&str` to `PascalCase` `String` while respecting identifier rules
+pub fn safe_pascal_case(ident: &str) -> String {
+    safe_identifier_name(ident.to_pascal_case())
+}
+
+/// respects identifier rules, such as, an identifier must not start with a numeric char
+fn safe_identifier_name(name: String) -> String {
+    if name.starts_with(|c: char| c.is_numeric()) {
+        format!("_{}", name)
+    } else {
+        name
+    }
+}
+
+/// Expands an identifier as snakecase and preserve any leading or trailing underscores
+pub fn safe_snake_case_ident(name: &str) -> Ident {
+    let i = name.to_snake_case();
+    ident(&preserve_underscore_delim(&i, name))
+}
+
+/// Expands an identifier as pascal case and preserve any leading or trailing underscores
+pub fn safe_pascal_case_ident(name: &str) -> Ident {
+    let i = name.to_pascal_case();
+    ident(&preserve_underscore_delim(&i, name))
+}
+
 /// Reapplies leading and trailing underscore chars to the ident
 /// Example `ident = "pascalCase"; alias = __pascalcase__` -> `__pascalCase__`
 pub fn preserve_underscore_delim(ident: &str, alias: &str) -> String {
@@ -65,19 +95,18 @@ where
     S: AsRef<str>,
 {
     let address_str = address_str.as_ref();
-    if !address_str.starts_with("0x") {
-        return Err(anyhow!("address must start with '0x'"))
-    }
+    eyre::ensure!(address_str.starts_with("0x"), "address must start with '0x'");
     Ok(address_str[2..].parse()?)
 }
 
 /// Perform an HTTP GET request and return the contents of the response.
+#[cfg(not(target_arch = "wasm32"))]
 pub fn http_get(_url: &str) -> Result<String> {
-    cfg_if! {
-        if #[cfg(any(target_arch = "wasm32", not(feature = "reqwest")))]{
-            Err(anyhow!("HTTP is unsupported"))
-        } else {
+    cfg_if::cfg_if! {
+        if #[cfg(feature = "reqwest")]{
             Ok(reqwest::blocking::get(_url)?.text()?)
+        } else {
+            eyre::bail!("HTTP is unsupported")
         }
     }
 }
@@ -98,7 +127,7 @@ pub fn resolve_path(raw: &str) -> Result<PathBuf> {
                 unprocessed = rest;
             }
             None => {
-                anyhow::bail!("Unable to parse a variable from \"{}\"", tail)
+                eyre::bail!("Unable to parse a variable from \"{}\"", tail)
             }
         }
     }
@@ -139,6 +168,17 @@ fn take_while(s: &str, mut predicate: impl FnMut(char) -> bool) -> (&str, &str)
     s.split_at(index)
 }
 
+/// Returns a list of absolute paths to all the json files under the root
+pub fn json_files(root: impl AsRef<std::path::Path>) -> Vec<PathBuf> {
+    walkdir::WalkDir::new(root)
+        .into_iter()
+        .filter_map(Result::ok)
+        .filter(|e| e.file_type().is_file())
+        .filter(|e| e.path().extension().map(|ext| ext == "json").unwrap_or_default())
+        .map(|e| e.path().into())
+        .collect()
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -169,7 +209,7 @@ mod tests {
     #[test]
     fn parse_address_missing_prefix() {
         assert!(
-            !parse_address("0000000000000000000000000000000000000000").is_ok(),
+            parse_address("0000000000000000000000000000000000000000").is_err(),
             "parsing address not starting with 0x should fail"
         );
     }
@@ -177,7 +217,7 @@ mod tests {
     #[test]
     fn parse_address_address_too_short() {
         assert!(
-            !parse_address("0x00000000000000").is_ok(),
+            parse_address("0x00000000000000").is_err(),
             "parsing address not starting with 0x should fail"
         );
     }
